//	Copyright (c) 2008 itok ( http://itok.jp/ , http://110k.net/ )
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification, 
//	are permitted provided that the following conditions are met:
//
//	- Redistributions of source code must retain the above copyright notice, 
//	  this list of conditions and the following disclaimer.
//	- Redistributions in binary form must reproduce the above copyright notice, 
//	  this list of conditions and the following disclaimer in the documentation 
//	  and/or other materials provided with the distribution.
//	- Neither the name of itok nor the names of its contributors may be used to endorse 
//	  or promote products derived from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS 
//	AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
//	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
//	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "PhotoZouRequest.h"
#import "NSXMLElement-ex.h"
#import "NSData-Base64Extensions.h"

@interface PhotoZouRequest (Private) 
-(void) addBasicHeader;
@end

@implementation PhotoZouRequest

-(void) start
{
	[self addBasicHeader];
	[super start];
}

-(id) responseData
{
	NSData* data = [super responseData];
	if (data && [data isKindOfClass:[NSData class]]) {
		NSError* err;
		NSXMLDocument* xml = [[[NSXMLDocument alloc] initWithData:data options:0 error:&err] autorelease];
		if (!xml) {
			NSLog(@"%@", [err description]);
		}
		return xml;
	}
	return nil;
}

-(NSError*) lastError
{
	NSXMLDocument* xml = [self responseData];
	if (!(xml && [xml isKindOfClass:[NSXMLDocument class]])) {
		xml = [super responseData];
	}
	if (xml && [xml isKindOfClass:[NSXMLDocument class]]) {
		NSXMLElement* root = [xml rootElement];
		if ([[root attributeValueForXPath:@"/rsp" andName:@"stat" error:nil] isEqualTo:@"fail"]) {
			id code = [root attributeValueForXPath:@"/rsp/err" andName:@"code" error:nil];
			id msg = [root attributeValueForXPath:@"/rsp/err" andName:@"msg" error:nil];
			if (code) {
				NSDictionary* userInfo = (msg) ? [NSDictionary dictionaryWithObjectsAndKeys:msg, NSLocalizedDescriptionKey, nil] : nil;
				NSError* err = [NSError errorWithDomain:PURequestErrorDomain code:[code intValue] userInfo:userInfo];
				return err;
			}
		}
	}
	return nil;
}

-(void) setValue:(id)value forUndefinedKey:(id)key
{
}

-(void) setNilValue:(id)value forKey:(id)key
{
}

-(id) valueForUndefinedKey:(id)key
{
	return nil;
}

@end

@implementation PhotoZouRequest (Private)

-(void) addBasicHeader
{
	NSData* baseData = STR_TO_DATA(([NSString stringWithFormat:@"%@:%@", [PURequest username], [PURequest password]]));
	NSString* baseStr = [baseData encodeBase64WithNewlines:NO];
	CFHTTPMessageSetHeaderFieldValue(m_message, CFSTR("Authorization"), (CFStringRef)[NSString stringWithFormat:@"Basic %@", baseStr]);
}

@end
