//	Copyright (c) 2008 itok ( http://itok.jp/ , http://110k.net/ )
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification, 
//	are permitted provided that the following conditions are met:
//
//	- Redistributions of source code must retain the above copyright notice, 
//	  this list of conditions and the following disclaimer.
//	- Redistributions in binary form must reproduce the above copyright notice, 
//	  this list of conditions and the following disclaimer in the documentation 
//	  and/or other materials provided with the distribution.
//	- Neither the name of itok nor the names of its contributors may be used to endorse 
//	  or promote products derived from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS 
//	AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
//	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
//	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "PhotoZouService.h"
#import "PhotoZou.h"
#import "PUMacros.h"

@interface PhotoZouService (Private)

-(void) getAlbum;
-(void) clearAlbum;

@end

@implementation PhotoZouService

-(id) init
{
	if (self = [super init]) {
		[NSBundle loadNibNamed:@"PhotoZou" owner:self];
		m_albumDic = [[NSMutableDictionary alloc] init];
		m_activated = NO;
		
		m_usernameKey = [[NSString alloc] initWithString:@"PhotoZou_Username"];
		m_keychainURL = [PHOTOZOU_URL retain];
		
		[self setupAccount];
	}
	return self;
}

-(void) dealloc
{
	[m_albumDic release];
	
	[super dealloc];
}

#pragma mark ___ PUService protocol ___

-(NSString*) title
{
	return LOCALIZED(@"PhotoZou");
}

-(NSArray*) requestsWithMgr:(ExportMgr*)mgr andURLs:(NSDictionary*)urlDic
{
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	int i;
	PhotoZou_PhotoAdd* req;
	NSMutableArray* reqs = [NSMutableArray array];
	NSString* origPath;
	NSURL* url;
	for (i = 0; i < [mgr imageCount]; i++) {
		req = [[PhotoZou_PhotoAdd alloc] init];
		origPath = [mgr imagePathAtIndex:i];
		if (urlDic && (url = [urlDic objectForKey:origPath])) {
		} else {
			url = [NSURL fileURLWithPath:origPath];
		}
		[req setValue:url forKey:@"fileUrl"];
		[req setValue:[[m_albumDic objectForKey:[_albumMenu objectAtIndex:_albumIndex]] valueForKey:@"album_id"] forKey:@"albumId"];
		// フォト蔵は jpeg, png, gif サポートなので jpeg だけでよいのでは
		[req setValue:@"image/jpeg" forKey:@"contentType"];
		if ([defaults boolForKey:@"PhotoZou_UseTitle"]) {
			[req setValue:[mgr imageTitleAtIndex:i] forKey:@"name"];
		}
		if ([defaults boolForKey:@"PhotoZou_UseComment"]) {
			[req setValue:[mgr imageCommentsAtIndex:i] forKey:@"comment"];
		}
		if ([defaults boolForKey:@"PhotoZou_UseKeyword"]) {
			[req setValue:[mgr imageKeywordsAtIndex:i] forKey:@"tags"];
		}
		[reqs addObject:req];
	}
	return reqs;
}

-(NSDictionary*) convertAttributes
{
	return nil;
}

-(NSURL*) URLForLastUpload:(NSArray*)responses
{
	NSString* path;
	PZAlbum* album = [m_albumDic objectForKey:[_albumMenu objectAtIndex:_albumIndex]];
	if ([responses count] == 0) {
		path = [NSString stringWithFormat:@"%@photo/list/%@", PHOTOZOU_PATH, [album valueForKey:@"user_id"]];
	} else {
		if ([responses count] == 1) {
			PZPhoto* photo = [responses objectAtIndex:0];
			if ([photo isKindOfClass:[PZPhoto class]]) {
				/* 写真ページそのもの */
				path = [photo valueForKey:@"url"];
			}
		} else {
			/* アルバムページ */
			path = [NSString stringWithFormat:@"%@photo/list/%@/%@", PHOTOZOU_PATH, [album valueForKey:@"user_id"], [album valueForKey:@"album_id"]];
		}
	}
	if (path) {
		return [NSURL URLWithString:path];
	}
	return nil;
}

-(void) login
{
	if (IS_NULL_STR(_username) || IS_NULL_STR(_password)) {
		[self clearAlbum];
		return;
	}
	[PhotoZouRequest setUsername:_username];
	[PhotoZouRequest setPassword:_password];
	[self getAlbum];
}

#pragma mark ___ delegate ___

-(void) request:(PURequest*)request didCompletedWithResult:(int)code
{
//	NSLog(@"didCompletedWithResult %d", code);
	if ([request isKindOfClass:[PhotoZou_PhotoAlbum class]]) {
		NSError* err = [request lastError];
		if (err) {
			NSRunAlertPanel([NSString stringWithFormat:LOCALIZED(@"Error Occurred : %d"), [err code]], [err localizedDescription], LOCALIZED(@"OK"), @"", @"");
			[request release];
			return;
		}
		
		NSArray* albums = [request responseData];
		NSEnumerator* enume = [albums objectEnumerator];
		PZAlbum* album;
		[m_albumDic removeAllObjects];
		while (album = [enume nextObject]) {
			[m_albumDic setObject:album forKey:[album valueForKey:@"name"]];
		}
		[self setValue:[m_albumDic allKeys] forKey:@"albumMenu"];
//		NSLog(@"%@", [_albumMenu description]);
	}
	
	[request release];
}
@end

@implementation PhotoZouService (Private)

-(void) getAlbum
{
	PhotoZou_PhotoAlbum* album = [[PhotoZou_PhotoAlbum alloc] init];
	[album setDelegate:self];
	[album start];
}	

-(void) clearAlbum
{
	[m_albumDic removeAllObjects];
	[self setValue:[NSArray array] forKey:@"albumMenu"];
}

@end

